use Test2::V0;

use JSON::XS ();
use JSON::Types ();

use Cpanel::JSON::XS ();
use Cpanel::JSON::XS::Type;

my $items = [
  {
    name => 'foo',
    details => [
      { id => 101, timestamp => 11111 },
      { id => 102, timestamp => 11112 },
    ],
  },
  {
    name => 'bar',
    details => [
      { id => 201, timestamp => 22222 },
      { id => 202, timestamp => 22223 },
      { id => 203, timestamp => 22224 },
    ],
  },
];

my $json_types_result;
subtest 'JSON::Typesの場合' => sub {
  my @json_items;
  for my $item ($items->@*) {
    my @json_details;
    for my $detail ($item->{details}->@*) {
      push @json_details => {
        id        => JSON::Types::number $detail->{id},
        timestamp => JSON::Types::number $detail->{timestamp},
      }
    }

    push @json_items => {
      name    => JSON::Types::string $item->{name},
      details => \@json_details,
    }
  }

  my $json_xs = JSON::XS->new->canonical;
  $json_types_result = $json_xs->encode(\@json_items);
  note $json_types_result;
  pass;
};


my $cpanel_json_result;
subtest 'Cpanel::JSON::XS::Typeの場合' => sub {

  # 期待するJSONの構造を宣言
  use constant ItemDetail => {
    id        => JSON_TYPE_INT,
    timestamp => JSON_TYPE_INT,
  };

  # Itemは、ItemDetailのリストをネストしている
  use constant Item => {
    name    => JSON_TYPE_STRING,
    details => json_type_arrayof(ItemDetail),
  };

  # 取得してきたデータ($items)を
  # 型と一緒に渡してJSONエンコードする
  my $cpanel_json = Cpanel::JSON::XS->new->canonical;
  $cpanel_json_result = $cpanel_json->encode(
    $items,
    json_type_arrayof(Item)
  );

  note $cpanel_json_result;
  pass;
};

ok $json_types_result eq $cpanel_json_result,
  'JSON::TypesとCpanel::JSON::XS::Typeの結果が一致';

done_testing;
